#!/usr/bin/env python
"""
Module TokenConverter
Sub-Package CLASSES of Package PLIB
Copyright (C) 2008 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the TokenConverter class.
"""

from plib.stdlib import abstractsequence

class TokenConverter(abstractsequence):
    """
    Sequence class that initializes itself from a list of string tokens;
    it uses a list of tuples (the converters field) to convert each token to
    the appropriate data type and store it in a named attribute. The data
    can then be accessed as a sequence of string tokens, or through each
    named attribute as the appropriate native data type. The purpose is to
    automate the storage and retrieval of the data from simple text files
    as string tokens, while allowing native-type operations on the data
    (such as arithmetic on numbers and dates) to be easily done while the
    data is loaded.
    
    The converters class field in subclasses must be filled with a list of
    2-tuples of the form (attr_name, convert_func), giving the name of the
    attribute of this object that will store the data, and the data type
    conversion function which takes a single string argument and returns
    the value converted to the desired data type.
    
    Note that there is no length-checking on the list of tokens passed to
    the constructor; passing a list of tokens that is longer than the list
    of converters will result in an IndexError.
    """
    
    converters = []
    
    def __init__(self, tokens=[]):
        for index, token in enumerate(tokens):
            self[index] = token
    
    def __len__(self):
        return len(self.converters)
    
    def __getitem__(self, index):
        return str(getattr(self, self.converters[index][0]))
    
    def __setitem__(self, index, value):
        setattr(self, self.converters[index][0], self.converters[index][1](value))
